const {constants, log, helpers} = require('utils-nxg-cg');
const {objectTransformCsv, objectTransformJson, extraProp, separators} = require('./objects');
const {transformJsonToCSV, transformCsvToJson} = require('./transforms');
const {isObjectValid} = require("utils-nxg-cg/utils/helpers");

const invalid = `Invalid separator, you must use only these separators ${Object.keys(separators).toString()}`;

/**
 * Method for process the data user and execute query single or batch by transactions
 * @param msg
 * @param cfg
 * @param test
 * @returns {Promise<array|object>}
 */
const transformJsonCsv = async (msg = {}, cfg = {}, test = false) => {
    try {
        const {data} = msg;

        let properties = {...objectTransformCsv};
        let extraProps = {...extraProp};
        if (!test && !data) {
            throw Error(`${constants.ERROR_PROPERTY} data`);
        }
        const valid = helpers.validProperties(properties, data, cfg);

        if (valid) {
            helpers.validProperties(extraProps, data, cfg, true);
            properties = {...properties, ...extraProps};
            if(!isObjectValid(properties.content))
                throw Error(constants.ERROR_JSON_FORMAT);
            if (!validSeparator(properties)) throw Error(invalid);
            return await transformJsonToCSV(properties);
        }
    } catch (e) {
        log.error(e);
        throw Error(e.toString());
    }
};

/**
 * Method to transform csv files to json
 * @param msg
 * @param cfg
 * @param test
 * @returns {Promise<any|undefined>}
 */
const transformCsvJson = async (msg = {}, cfg = {}, test = false) => {
    try {
        const {data} = msg;

        let properties = {...objectTransformJson};
        let extraProps = {...extraProp};

        if (!test && !data) {
            throw Error(`${constants.ERROR_PROPERTY} data`);
        }
        const valid = helpers.validProperties(properties, data, cfg);

        if (valid) {
            helpers.validProperties(extraProps, data, cfg, true);
            properties = {...properties, ...extraProps};
            if (!validSeparator(properties)) throw Error(invalid);
            if (!helpers.validateBase64(properties.content))
                throw Error(`${constants.ERROR_WITH_PROPERTY} content is not base64`);

            return await transformCsvToJson(properties);
        }
    } catch (e) {
        log.error(e);
        throw Error(e.toString());
    }
};

/**
 * function to valid separators with custom and default headers and without it
 * @param properties
 * @returns {boolean}
 */
function validSeparator (properties){
    let temp = properties.delimiter;
    properties.delimiter = separators[properties.delimiter];
    if(properties.headers && temp === '$')
        properties.delimiter = '$';
    if(properties.headers && temp === '*')
        properties.delimiter = '*';
    if(properties.headers && temp === '|')
        properties.delimiter = '|';
    if(properties.headers && temp === '.')
        properties.delimiter = '.';
    return !!properties.delimiter;
}

module.exports = {transformJsonCsv, transformCsvJson};